package sample;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.stage.Stage;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;

public class Controller {

    public String inputText;
    public String optimData;

    @FXML
    public Button submitBtn;
    public Label msgLabel;
    public Button clearBtn;
    public TextArea inputBox;
    public TableView tableFormat;
    public TableColumn alleleCol;
    public TableColumn peptideCol;
    public Button optimScreenBtn;

    @FXML
    void clearInput(ActionEvent event) {
        inputText = "";
        optimData = "";
        inputBox.clear();
        msgLabel.setText("");
        tableFormat.getItems().clear();
    }

    public void submitInput(ActionEvent actionEvent) {
        if(inputBox.getText().equals(""))
        {
            msgLabel.setText("Please enter data into the text box");
        }
        else
        {
            if(checkFormat())
            {
                inputText = inputBox.getText();
                populateTable(inputText);
                optimData = optimCalc(inputText);
             }
        }
    }

    private void populateTable(String data) {

        String[] rowArray = data.split("\\r?\\n");
        String[] alleles = new String[rowArray.length];
        String[] epitopes = new String[rowArray.length];
        ObservableList<PeptideCombo> tableData = FXCollections.observableArrayList();


        for(int i = 0; i < rowArray.length; i++)
        {
            alleles[i] = rowArray[i].substring(0,11);
            epitopes[i] = rowArray[i].substring(rowArray[i].lastIndexOf("\t")+1);
        }

        for(int i = 0; i < alleles.length; i++)
        {
            tableData.add(new PeptideCombo(epitopes[i],alleles[i]));
        }

        alleleCol.setCellValueFactory(new PropertyValueFactory<>("Allele"));
        peptideCol.setCellValueFactory(new PropertyValueFactory<>("Peptide"));
        tableFormat.setItems(tableData);
    }

    private boolean checkFormat() {
        return true;
    }

    private String optimCalc(String data) {
        String[] rowArray = data.split("\\r?\\n");
        String[] alleles = new String[rowArray.length];
        String[] epitopes = new String[rowArray.length];

        for(int i = 0; i < rowArray.length; i++)
        {
            alleles[i] = rowArray[i].substring(0,11);
            epitopes[i] = rowArray[i].substring(rowArray[i].lastIndexOf("\t")+1);
        }

        String[] epitopesNR = new HashSet<String>(Arrays.asList(epitopes)).toArray(new String[0]);
        String[][] alleleList = new String[epitopesNR.length][0];

        for(int i = 0; i < epitopesNR.length; i++)
        {
            String tempEpi = epitopesNR[i];

            for(int w = 0; w < epitopes.length; w++)
            {
                if(epitopes[w].equals(tempEpi))
                {
                    String[] tempArray = new String[alleleList[i].length+1];
                    for(int k = 0; k < alleleList[i].length; k++)
                    {
                        tempArray[k] = alleleList[i][k];
                    }
                    tempArray[tempArray.length-1] = alleles[w];
                    alleleList[i] = tempArray;
                }
            }
        }

        ArrayList<String> optimEpi = new ArrayList<String>();
        HashSet<String> allelesNRARRAY = new HashSet<String>(Arrays.asList(alleles));
        ArrayList<String> allelesNR = new ArrayList<String>();
        for(String s:allelesNRARRAY)
        {
            if(!allelesNR.contains(s))
            {
                allelesNR.add(s);
            }
        }
        ArrayList <String[]> optimAllele = new ArrayList<String[]>();

        String searchAllele;
        ArrayList<Integer> matchIndex = new ArrayList<Integer>();
        int index = 0;
        int temp = 0;

        while(allelesNR.size() > 0)
        {
            searchAllele = allelesNR.get(0);
            for(int i = 0; i < alleleList.length; i++)
            {
                for(int w = 0; w < alleleList[i].length; w++)
                {
                    if(alleleList[i][w].equals(searchAllele))
                    {
                        matchIndex.add(i);
                    }
                }
            }
            for(int i = 0; i < matchIndex.size(); i++)
            {
                if(alleleList[matchIndex.get(i)].length > temp)
                {
                    temp = alleleList[matchIndex.get(i)].length;
                    index = matchIndex.get(i);
                }
            }
            optimEpi.add(epitopesNR[index]);
            optimAllele.add(alleleList[index]);

            for(String s:alleleList[index])
            {
                allelesNR.remove(s);
            }
            matchIndex.clear();
            index = 0;
            temp = 0;
        }

        String returnString = "";

        for(int i = 0; i < optimEpi.size(); i++)
        {
            returnString += optimEpi.get(i) + "\t";
            for(int w = 0; w < optimAllele.get(i).length-1; w++)
            {
                returnString += optimAllele.get(i)[w] + ",";
            }
            returnString += optimAllele.get(i)[optimAllele.get(i).length-1] + "\n";
        }

        return returnString;
    }

    public void openScreen(ActionEvent actionEvent) throws Exception {
        FXMLLoader loader = new FXMLLoader(getClass().getResource("optimScreen.fxml"));
        Parent root = loader.load();
        OptimScreenController optimScreenController = loader.getController();
        optimScreenController.setOptimText(optimData);
        optimScreenController.setInputText(inputText);
        optimScreenBtn.getScene().setRoot(root);
    }

    public void setInputText(String input) {
        inputBox.setText(input);
    }
}
